INSTALLER_BIND_MOUNTS="/dev /sys /proc"
INSTALLER_BIND_UMOUNTS="/proc /sys /dev"

# -------------------------------------------------------------------
# Logging

log() {
    echo "$@" >> $INSTALLER_LOGFILE
}

exec_log() {
    echo "> $@" >> $INSTALLER_LOGFILE
    "$@" &>> $INSTALLER_LOGFILE
}

exec_log_err() {
    echo "> $@" >> $INSTALLER_LOGFILE
    "$@" 2>> $INSTALLER_LOGFILE
}

upload_logfile() {
    curl -s --capath ./certs -H "Content-Type: text/plain" -T "$INSTALLER_LOGFILE" -m 20 "$INSTALLER_LOGFILE_URL" > /dev/null
}

# -------------------------------------------------------------------
# UI

# Nachricht ohne OK-Button
dialog_info() {
    log ">>> $1"
    dialog --title "SDM-Installation" --infobox "$1" 15 70 || exit 1
}

# Nachricht mit OK-Button
dialog_msg() {
    log ">>> $1"
    dialog --title "SDM-Installation" --msgbox "$1" 15 70 || exit 1
}

dialog_ask_password() {
    dialog --title "SDM-Installation" --insecure --nocancel --passwordbox "Passwort:" 15 70
}

fatal_error() {
    upload_logfile
    
    log "!!! Installation abgebrochen: "
    dialog_msg "$@ Bestätigen, um Rechner neuzustarten."
    
    exit 1
}

# -------------------------------------------------------------------
# Bind mounts

bind_mount() {
    for DISKS_PARTITIONS_FS in $INSTALLER_BIND_MOUNTS
    do
        mount --bind $DISKS_PARTITIONS_FS "$target$DISKS_PARTITIONS_FS"
    done
}

bind_umount() {
    for DISKS_PARTITIONS_FS in $INSTALLER_BIND_UMOUNTS
    do
        umount "$target$DISKS_PARTITIONS_FS"
    done
}


# -------------------------------------------------------------------
# Paritionierung
# neue Partitionstabelle anlegen

part_clean() {
    # Anfang der Festplatte überschreiben, um GPT Tabelle wegzubekommen
    exec_log dd if=/dev/zero of="$1" bs=1M count=16
    sleep 2
    part_rescan
    sleep 1
    
    if [ "$part" == "msdos" ]
    then
        exec_log /usr/local/sbin/fdisk "$1" <<EOF
o
w
EOF
    elif [ "$part" == "gpt" ]
    then
        # EFI-Systempartition anlegen ...
        exec_log /usr/local/sbin/fdisk "$1" <<EOF
g
w
EOF
    fi
}

# neue Partition anlegen
# $2 = Partitionsnummer
# $3 = Start der Partition
# $4 = Ende der Partition (mit + am Anfang: Größe der Partition)
part_new() {
    exec_log /usr/local/sbin/fdisk -u "$1" <<EOF
n
p
$2
$3
$4
w
EOF
}

part_rescan() {
    sleep 2
    exec_log partprobe
    sleep 2
}

# -------------------------------------------------------------------
# Partitionierung
# Partitionen anlegen

frz_partition() {
    # Partionstabelle erstellen und Partitionen anlegen ohne Fehlerkontrolle.
    # manchmal kommt ein Fehler, dass die Partitionstabelle nicht neu
    # eingelesen werden konnte. Das ist aber so ziemlich egal, kann aber
    # nicht von anderen Fehlern unterschieden werden.
    
    if [ "$partclean" == "yes" ]
    then
        dialog_info "Erstelle neue Partitionstabelle auf $disk ..."
        sleep 2
        part_clean "$disk"
        part_rescan
    fi
    
    if [ -n "$rootpart" ]
    then
        dialog_info "Erstelle Root-Partition als Partition $rootpart auf $disk"
        sleep 1
    
        if [ -n "$rootsize" ]
            then part_new "$disk" "$rootpart" "$rootstart" "$rootsize"
            else part_new "$disk" "$rootpart" "$rootstart" "$rootend"
        fi
    fi
    
    if [ -n "$swappart" ]
    then
        dialog_info "Erstelle Swap-Partition als Partition $swappart auf $disk"
        sleep 1
        if [ -n "$swapsize" ]
            then part_new "$disk" "$swappart" "$swapstart" "$swapsize"
            else part_new "$disk" "$swappart" "$swapstart" "$swapend"
        fi
    fi
    
    part_rescan
}


# -------------------------------------------------------------------
# Formatierung

frz_format() {
    if [ -n "$rootpart" ]
    then
        dialog_info "Formatiere Root-Partition ${disk}${rootpart} mit ext4"
        sleep 1
    
        exec_log mkfs.ext4 -q "${disk}${rootpart}" || fatal_error "Formatierung der Root-Partition fehlgeschlagen!"
    fi
    
    if [ -n "$swappart" ]
    then
        dialog_info "Formatiere Swap-Partition ${disk}${swappart}"
        sleep 1
        exec_log mkswap "${disk}${swappart}" || fatal_error "Formatierung der Swap-Partition fehlgeschlagen!"
    fi
}


# -------------------------------------------------------------------
# Mount

frz_mount() {
    dialog_info "Mounte Root-Partition ${disk}${rootpart} auf $target"
    if [ -n "$rootpart" ]
    then
        if [ -z "$roottype" ]
        then
            roottype="ext4"
        fi
        mkdir -p $target
        mount -t "$roottype" "${disk}${rootpart}" "$target" || fatal_error "Das Ziel-Dateisystem konnte nicht eingehangen werden!"
    fi
}


# -------------------------------------------------------------------
# Installation

sdm_install() {
    frz_log_vars
    frz_pre
    frz_partition
    frz_format
    frz_mount
    frz_sync
    frz_hostname
    frz_bootmgr
    frz_provision
    frz_post
    frz_cleanup
}


# -------------------------------------------------------------------
# 

frz_log_vars() {
    log hostname=$hostname
    log force_hostname=$force_hostname
    log target=$target
    log source=$source
    log config=$config
    log disk=$disk
    log part=$part
    log efidisk=$efidisk
    log efipart=$efipart
    log partclean=$partclean
    log rootpart=$rootpart
    log rootsize=$rootsize
    log rootstart=$rootstart
    log rootend=$rootend
    log swappart=$swappart
    log swapsize=$swapsize
    log swapstart=$swapstart
    log swapend=$swapend
    log subdir=$subdir
}


# -------------------------------------------------------------------
# 

frz_pre() {
    :
}


# -------------------------------------------------------------------
# 

frz_post() {
    :
}



# -------------------------------------------------------------------
# 

frz_hostname() {
    if [ -n "$force_hostname" ]
    then
        hostname="$force_hostname"
    fi
    
    if [ -n "$target" ] && [ -n "$hostname" ]
    then
        dialog_info "Setze Hostname auf $hostname"
        sleep 1
        
        echo $hostname > "$target/etc/hostname"
    fi
}


# -------------------------------------------------------------------
# 

provision() {
    if [ -n "$provision" ]
    then
        dialog_info "Provisioniere System mit '$provision' ..."
        bind_mount
        exec_log chroot "$target" /bin/bash -c "cd /etc/ansible; ansible-playbook ${provision}.yml -i hosts --limit local"
        bind_umount
    fi
}


# -------------------------------------------------------------------
# 

frz_cleanup() {
    dialog_info "Führe ldconfig aus ..."
    exec_log chroot $target ldconfig
    
    umount $target
    rmdir $target
    
    return 0
}
